part of google_oauth2_browser;

typedef void _ProxyCallback(String subject, List<String> args);

/// Sets up a channel for listening to the token information posted by the
/// authorization url using the postMessage flow.
///
/// We create a hidden iframe hosting the provider's 'postmessageRelay' page,
/// which receives token information from the authorization popup and posts
/// it to this document. We also add a message listener to this document.
/// It detects such messages and invokes the provided callback.
class _ProxyChannel {
  String _nonce;
  String _provider;
  String _expectedOrigin;
  IFrameElement _element;
  _ProxyCallback _callback;
  StreamSubscription _streamsub;

  _ProxyChannel(String this._provider, _ProxyCallback this._callback) {
    _nonce = (0x7FFFFFFF & _random()).toString();
    _expectedOrigin = _origin(_provider);
    _element = _iframe(_getProxyUrl());
    _streamsub = window.onMessage.listen(_onMessage);
  }

  void close() {
    _element.remove();
    _streamsub.cancel();
  }

  void _onMessage(MessageEvent event) {
    if (event.origin != _expectedOrigin) {
      print("Invalid message origin: ${event.origin} / Expected ${_expectedOrigin}");
      return;
    }
    var data;
    try {
      data = JSON.decode(event.data);
    } catch (e) {
      print("Invalid JSON received via postMessage: ${event.data}");
      return;
    }
    if (!(data is Map) || (data['t'] != _nonce)) {
      return;
    }
    String subject = data['s'];
    if (subject.endsWith(':$_nonce')) {
      subject = subject.substring(0, subject.length - _nonce.length - 1);
    }
    _callback(subject, data['a'] as List<String>);
  }

  /// Computes the javascript origin of an absolute URI.
  String _origin(String uriString) {
    final uri = Uri.parse(uriString);
    var portPart;
    if (uri.port == 0 || (uri.port == 443 && uri.scheme == "https")) {
      portPart = "";
    } else {
      portPart = ":${uri.port}";
    }
    return "${uri.scheme}://${uri.host}$portPart";
  }

  String _getProxyUrl() {
    Map<String, String> proxyParams = {"parent": window.location.origin};
    String proxyUrl = UrlPattern.generatePattern("${_provider}postmessageRelay",
        {}, proxyParams);
    return Uri.parse(proxyUrl)
        .resolve("#rpctoken=$_nonce&forcesecure=1").toString();
  }
}